@@ -33,13 +33,6 @@ either expressed or implied, of the Regents of The University of Michigan.
 #include "g2d.h"
 #include "common/math_util.h"
 
-#ifdef _WIN32
-static inline long int random(void)
-{
-        return rand();
-}
-#endif
-
 double g2d_distance(const double a[2], const double b[2])
 {
     return sqrtf(sq(a[0]-b[0]) + sq(a[1]-b[1]));
@@ -127,7 +120,7 @@ int g2d_polygon_contains_point_ref(const zarray_t *poly, double q[2])
 
     double acc_theta = 0;
 
-    double last_theta;
+    double last_theta = 0;
 
     for (int i = 0; i <= psz; i++) {
         double p[2];
@@ -328,7 +321,7 @@ int g2d_polygon_contains_point(const zarray_t *poly, double q[2])
     int psz = zarray_size(poly);
     assert(psz > 0);
 
-    int last_quadrant;
+    int last_quadrant = 0;
     int quad_acc = 0;
 
     for (int i = 0; i <= psz; i++) {
@@ -725,6 +718,13 @@ int g2d_polygon_rasterize(const zarray_t *poly, double y, double *x)
 */
 #if 0
 
+#ifdef _WIN32
+static inline long int random(void)
+{
+        return rand();
+}
+#endif
+
 #include "timeprofile.h"
 
 int main(int argc, char *argv[])
