@@ -51,18 +51,20 @@ matd_t *matd_create(int rows, int cols)
     if (rows == 0 || cols == 0)
         return matd_create_scalar(0);
 
-    matd_t *m = calloc(1, sizeof(matd_t) + (rows*cols*sizeof(double)));
+    matd_t *m = calloc(1, sizeof(matd_t));
     m->nrows = rows;
     m->ncols = cols;
+    m->data = calloc(rows * cols, sizeof(double));
 
     return m;
 }
 
 matd_t *matd_create_scalar(TYPE v)
 {
-    matd_t *m = calloc(1, sizeof(matd_t) + sizeof(double));
+    matd_t *m = calloc(1, sizeof(matd_t));
     m->nrows = 0;
     m->ncols = 0;
+    m->data = calloc(1, sizeof(double));
     m->data[0] = v;
 
     return m;
@@ -220,7 +222,8 @@ void matd_destroy(matd_t *m)
     if (!m)
         return;
 
-    assert(m != NULL);
+    assert(m->data != NULL);
+    free(m->data);
     free(m);
 }
 
@@ -402,7 +405,7 @@ double matd_det_general(const matd_t *a)
     // The determinant of a can be calculated as
     //     epsilon*det(L)*det(U),
     // where epsilon is just the sign of the corresponding permutation
-    // (which is +1 for an even number of permutations and is −1
+    // (which is +1 for an even number of permutations and is -1
     // for an uneven number of permutations).
     double det = mlu->pivsign * detL * detU;
 
